# Define the size of the set S
n = 8
S = list(range(n))

from itertools import product

all_tables = product(S, repeat=n*n)

def tuple_to_table(t):
    return [list(t[i*n:(i+1)*n]) for i in range(n)]

# Function to check equation 854: For all x, y, z: x = x ◇ ((y ◇ z) ◇ (x ◇ z))
def check_equation_854(table):
    for x in S:
        for y in S:
            for z in S:
                yz = table[y][z]
                xz = table[x][z]
                yz_xz = table[yz][xz]
                lhs = x
                rhs = table[x][yz_xz]
                if lhs != rhs:
                    return False
    return True

# Function to check equation 413: For all x, y: x = x ◇ (x ◇ (x ◇ (y ◇ x)))
def check_equation_413(table):
    for x in S:
        for y in S:
            yx = table[y][x]
            x_yx = table[x][yx]
            x_x_yx = table[x][x_yx]
            rhs = table[x][x_x_yx]
            lhs = x
            if lhs != rhs:
                return True  # There exists x,y where equation 413 does not hold
    return False  # Equation 413 holds for all x,y

count = 0

for t in all_tables:
    table = tuple_to_table(t)
    if check_equation_854(table):
        if check_equation_413(table):
            # Found a table that satisfies equation 854 and does not satisfy equation 413
            count += 1
            print(f"--- Operation Table #{count} ---")
            # Print header
            header = "◇ | " + " ".join(str(elem) for elem in S)
            print(header)
            print("-" * len(header))
            # Print each row
            for x in S:
                row = f"{x} | " + " ".join(str(table[x][y]) for y in S)
                print(row)
            print("\n")

# If no tables found
if count == 0:
    print("No magma operation table satisfies the given conditions for S =", S)
else:
    print(f"Total operation tables found: {count}")
